package xyz.veronie.bgg.ui.parts;

import java.util.Arrays;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import xyz.veronie.bgg.result.ResultConfigManager;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.types.ResultAction;
import xyz.veronie.bgg.types.SourceFilter;
import xyz.veronie.bgg.ui.filters.AgeSourceFilter;
import xyz.veronie.bgg.ui.filters.BggUserSourceFilter;
import xyz.veronie.bgg.ui.filters.FamilySourceFilter;
import xyz.veronie.bgg.ui.filters.GeeklistSourceFilter;
import xyz.veronie.bgg.ui.filters.RankSourceFilter;
import xyz.veronie.bgg.ui.filters.SearchSourceFilter;
import xyz.veronie.bgg.ui.filters.SubtypeGroup;
import xyz.veronie.bgg.ui.filters.YearSourceFilter;



/// Has controls to download a list of thing IDs (+some meta info) from BGG.
/// There are different ways to configure which IDs to retrieve.
public class LoadFromBggPart {

	@Inject
	private ResultConfigManager configManager;
	
	@Inject private IEventBroker eventBroker;
	
	// inject all source filter composites
	@Inject private BggUserSourceFilter bggUserSourceFilter;
	@Inject private GeeklistSourceFilter geeklistSourceFilter;
	
	@PostConstruct
	public void createControls(Composite parent) {
		Composite main = new Composite(parent, SWT.FILL);
		main.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		
		GridLayout layout = new GridLayout(2, false);
		main.setLayout(layout);
		
		
		// contains configuration options for download of the bgg thing ID list
		Group dlConfigGroup = new Group(main, SWT.SHADOW_ETCHED_IN);
		GridData configGrid = new GridData(SWT.FILL, SWT.FILL, true, true);
		dlConfigGroup.setText("Configure download...");
		dlConfigGroup.setLayoutData(configGrid);
		
		GridLayout layoutG = new GridLayout(2, false);
		dlConfigGroup.setLayout(layoutG);
		
		Label lblSource = new Label(dlConfigGroup, SWT.LEFT);
		lblSource.setText("Source: ");
		lblSource.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false));
		
		// choose the bgg data source
		ComboViewer cbSource = new ComboViewer(dlConfigGroup, SWT.READ_ONLY);
		cbSource.setContentProvider(ArrayContentProvider.getInstance());

		List<SourceFilter> sources = Arrays.asList(new SourceFilter[] { 
				SourceFilter.BGG_USER, SourceFilter.GEEKLIST, SourceFilter.FAMILY, 
				SourceFilter.RANK, SourceFilter.YEAR, SourceFilter.AGE, SourceFilter.SEARCH });
		cbSource.setInput(sources);
		// TODO: implement all the sources

		cbSource.setSelection(new StructuredSelection(configManager.getResultConfig().source));
		// listener is configured further below

		
		// choose the bgg sub-site
		Label lblSubtype = new Label(dlConfigGroup, SWT.LEFT);
		lblSubtype.setText("Subtypes: ");
		lblSubtype.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false));
		
		SubtypeGroup gSubtypes = new SubtypeGroup();
		gSubtypes.create(dlConfigGroup, eventBroker, configManager);
		
		// choose action on result list
		Label lblAct = new Label(dlConfigGroup, SWT.LEFT);
		lblAct.setText("Action on result: ");
		lblAct.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false));
		
		ComboViewer cbAct = new ComboViewer(dlConfigGroup, SWT.READ_ONLY);
		cbAct.setContentProvider(ArrayContentProvider.getInstance());
		List<ResultAction> actions = Arrays.asList(new ResultAction[] { 
				ResultAction.ADD, ResultAction.REP, ResultAction.SUB,
				ResultAction.AND, ResultAction.MIS });
		cbAct.setInput(actions);
		cbAct.setSelection(new StructuredSelection(configManager.getResultConfig().action));
		cbAct.addSelectionChangedListener(new ISelectionChangedListener() {
			
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				eventBroker.send(EventConstants.TOPIC_ACTION_CHANGED, selection.getFirstElement());
			}
		});
		
		// area to display filter controls in
		Group gFilters = new Group(main, SWT.SHADOW_ETCHED_IN);
        gFilters.setText("");
        gFilters.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
        
		showFilter(gFilters, (IStructuredSelection)cbSource.getSelection()); 		// set content to initial selection
		cbSource.addSelectionChangedListener(new ISelectionChangedListener() {      // later, its content will be set via the dropdown
		    @Override
		    public void selectionChanged(SelectionChangedEvent event) {
		        IStructuredSelection selection = (IStructuredSelection) event.getSelection();
	        	// ASK: this is strange to me. Why is filterComposite known INSIDE a free function?
		        showFilter(gFilters, selection);
		        eventBroker.send(EventConstants.TOPIC_SOURCE_CHANGED, selection.getFirstElement());
		    }

		});
		
		
		Label empty = new Label(main, SWT.LEFT);
        empty.setText("");
        empty.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));        
        
		// Start retrieval when download button is pushed
		Button btDownload = new Button(main, SWT.PUSH);
		// TODO: how do I locate icons?
		try {
			Image dlIcon = new Image(getDisplay(), 
				getClass().getResourceAsStream("C:\\Users/goya/workspace/icons/icons/material_design/png/download-button.png"));
			btDownload.setImage(dlIcon);
		} 
		catch(Exception ex) {
			btDownload.setText("Download now");
		}
		btDownload.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
		        System.out.println("Downloading " + cbSource.getSelection().toString());

		        IStructuredSelection selection = (IStructuredSelection) cbSource.getSelection();
		        if(selection.size() == 0) return;
		        
		        if(selection.getFirstElement() == SourceFilter.BGG_USER) {
		        	String user = configManager.getResultConfig().user;
		        	if(user == null || user.isEmpty()) {
		        		System.out.println("Please enter a user name.");
		        		return;
		        	}
		        	System.out.println("...for user '" + user + "'");
		        } else if(selection.getFirstElement() == SourceFilter.GEEKLIST) {
		        	System.out.println("...for geeklist id '" + configManager.getResultConfig().geeklistId + "'");
		        } else if(selection.getFirstElement() == SourceFilter.FAMILY) {
		        	System.out.println("...for family id '" + FamilySourceFilter.getFamilyId() + "'");
		        }
		    }
		});
		btDownload.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false));

		main.pack();
		
	}
	
	
	/// show different filter controls depending on selection in cbSource ComboViewer
	public void showFilter(Composite parent, IStructuredSelection selection) {	
		SourceFilter elem = (SourceFilter) selection.getFirstElement();

		// clean up
		for(Control child : parent.getChildren()) {
			child.dispose();
		}
		
		// create a new filter area based on selection:
		switch(elem) {
		case BGG_USER:
			System.out.println("construct " + elem);
			bggUserSourceFilter.create(parent, SWT.FILL);
			break;
		case GEEKLIST:
			System.out.println("construct " + elem);
			geeklistSourceFilter.create(parent, SWT.FILL);
			break;
		case FAMILY:
			System.out.println("construct " + elem);
			FamilySourceFilter.create(parent, SWT.FILL);
			break;
		case RANK:
			RankSourceFilter.create(parent, SWT.FILL);
			break;
		case YEAR:
			YearSourceFilter.create(parent, SWT.FILL);
			break;
		case AGE:
			AgeSourceFilter.create(parent, SWT.FILL);
			break;
		case SEARCH:
			SearchSourceFilter.create(parent, SWT.FILL);
			break;
		default:
			System.out.println("construct " + elem + " (not implemented yet, try another one)");
		}
		
	}
	
	
	public static Display getDisplay() {
 	   Display display = Display.getCurrent();
 	   // may be null if outside the UI thread
	   if (display == null)
	      display = Display.getDefault();
	   return display;		
    }
	

	// DISPOSE images!
}
